/*
 Copyright 2002, 2003 Alexandre Gil
                                                                                
 This file is part of mwu-extractor.
                                                                                
 mwu-extractor is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
                                                                                
 mwu-extractor is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
                                                                                
 You should have received a copy of the GNU General Public License
 along with mwu-extractor; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Nome: Vector.h
// Objectivo: Interface da classe generica Vector
//            Inpirado no livro do Prof. Pedro Guerreiro (UNL/FCT)
//            Implenta um contentor de pointers para strings.
// Autor: Alexandre Gil
// Data: 23 Abr 2002

#ifndef _H_Vector
#define _H_Vector

#include<iostream>

#define VMED3(a,b,c) 																										\
   (items[a] < items[b] ? (items[b] < items[c] ? (b) : (items[a] < items[c] ?	(c) :	(a))) \
    							: (items[b] > items[c] ? (b) : (items[a] > items[c] ? (c) : (a))))																						                                                                                   

template <class T>
class Vector {
  private:
		T * items;
		unsigned long capacity;
		unsigned long filled;
		float growFactor;
		
		bool debug;
	private:
  	void TQuickSort(unsigned long l, unsigned long r); // pre r - l > 0
		unsigned long ChoosePivot(unsigned long l, unsigned long r) const;
	public:
		T& At(unsigned long x) const;   	 // pre x >= 0 && x < Capacity()
	  void PutAt(unsigned long x, const T& s); // pre x >= 0 && x <= Capacity()
	  void Add(const T& s); 
		void Copy(const Vector<T>& other);

		explicit Vector(unsigned long capacity);									// pre capacity > 0;
		explicit Vector(const Vector<T>& other);
		~Vector();

		Vector<T>& operator=(const Vector<T>& other);

		unsigned long Capacity() const;
		unsigned long Filled() const;

	  float GrowFactor() const;
	  void SetGrowFactor(float newGrowFactor);
	  void Grow();
	  void GrowTo(unsigned long newCapacity);

		void CleanLast();
		
		bool Has(const T& s) const;								
		unsigned long CountIf(const T& s) const;
		unsigned long IndexOf(const T& s) const;				   
		unsigned long IndexOfBin(const T& s) const;				   
		unsigned long LastIndexOf(const T& s) const;       

    void Swap(unsigned long x, unsigned long y);     							 // pre x >= 0 && x < Capacity()
    																																		   // pre y >= 0 && y < Capacity()

		void Reset();
		
		void Sort(unsigned long l, unsigned long r); // pre r - l > 0

    void Write(std::ostream& output = std::cout) const;

    void SetDebug(bool state);
};

#endif
