/*
 Copyright 2002, 2003 Alexandre Gil
                                                                                
 This file is part of mwu-extractor.
                                                                                
 mwu-extractor is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
                                                                                
 mwu-extractor is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
                                                                                
 You should have received a copy of the GNU General Public License
 along with mwu-extractor; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Nome: Corpus.h
// Objectivo: Interface da abstracao para ficheiros do Corpus
//            Recebe um corpus (nesta fase um ficheiro binario de n short int=2bytes)
// Autor: Alexandre Gil
// Data: 14 Jan 2002


#ifndef _H_Corpus
#define _H_Corpus

#include <fstream>
#include <iostream>

typedef unsigned int Token;
const int MAXFILES = 100;
const int MAXFILENAME = 200;

class Corpus
{
	public:
		class Pos {
			private:
				unsigned short fileNum;                        // deveria passar a unsigned short int
				unsigned long pos;
			public:
				Pos(const unsigned short fileNum, const unsigned long pos);
				Pos(const Pos& p);
				Pos();
				~Pos();

			  void Next(const Corpus& c);		      // pre: *this != c.LastPos()	
			  void Previous(const Corpus& c);		  // pre: *this != c.FirstPos()

        bool CanMoveSameFile(const Corpus& c, const int delta) const;
        unsigned long MaxMoveSameFile(const Corpus& c) const;
        void MoveSameFile(const int delta);   //pre CanMoveSameFile(c,delta)
        
				Pos& operator=(const Pos& p);
				bool operator<=(const Pos& p) const;
				bool operator>=(const Pos& p) const;
				bool operator<(const Pos& p) const;
				bool operator>(const Pos& p) const;
				bool operator==(const Pos& p) const;
				bool operator!=(const Pos& p)const ;
				void SetFilePos(unsigned long p);
				void SetFileNum(unsigned short p);
				unsigned long FilePos() const;
				unsigned short FileNum() const;
				
				friend std::ostream& operator<<(std::ostream& output, const Pos& p);
		};
		friend class Corpus::Pos;
	private:
		std::ifstream fList;
		char * fListName;
		struct {
			char *name;
			std::ifstream fp;
			unsigned long nTokens;
			Token * cache;
			unsigned long iBase;								// indice, dentro do Corpus, correspondente ao primeiro
																					// token deste ficheiro
		} file[MAXFILES];
		unsigned short numFiles;
		Pos pos;															 // Posicao no Corpus
		bool erro;
	private:
		void ReadCache(const unsigned short fNum);
		void ReadFileName(char * n, int max);
	public:
	  Corpus();
	  explicit Corpus(const char * fLName);
	  ~Corpus();
	  bool Erro() const;
	  Pos FirstPos() const;									// NumOfFiles() > 0
	  Pos LastPos() const;									// NumOfFiles() > 0
		Token At(Pos i) const; 								// pre DefinedPosition(i)
		bool DefinedPosition(Pos i) const;
		unsigned short NumOfFiles() const;
		unsigned long Dim() const;
		unsigned long Index(const Pos& p) const;
};


#endif
