/*
 Copyright 2002, 2003 Alexandre Gil
                                                                                
 This file is part of mwu-extractor.
                                                                                
 mwu-extractor is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
                                                                                
 mwu-extractor is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
                                                                                
 You should have received a copy of the GNU General Public License
 along with mwu-extractor; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Nome: Mask.cpp
// Objectivo: Implementacao da classe Mask.
// Autor: Alexandre Gil
// Data: 01 Fev 2002

#include "Mask.h"
#include "Globals.h"
#include "MyUtils.h"

#include <iostream>
#include <iomanip>

Byte Mask::first;

void Mask::SetFirst(const Byte newFirst)
{
	first = newFirst;
}

Byte Mask::At(const Byte x) const
{
	return value[x];
}

void Mask::PutAt(const Byte s, Byte x)  // s == GAP || s == FILLED != GAP
{
	if(value[x] != GAP && s == GAP)
	{
		value[x] = GAP;
		if(x == lenght -1)
		{
			short int i;
			for(i = lenght-1; i >= 0; i--)
				if(value[i] != GAP)
					break;
			lenght = i+1;
		}
		else
		{
			for(short int i=0; i < nTokens-1;i++)
				if(pFilled[i] >= x)
					pFilled[i] = pFilled[i+1];	
		}
		nTokens--;
	}
	else if(value[x] == GAP && s != GAP)
	{
		value[x] = s;
		if(x >= lenght)
		{
			lenght = x + 1;											// pressuposto que value[0]  sempre FILLED != GAP
			pFilled[nTokens] = x;
		}
		else
		{
			short int i,j;
			for(i=0; ;i++)
				if(pFilled[i] > x)
					break;
			for(j=nTokens; j > i;j--)
				pFilled[j] = pFilled[j-1];
			pFilled[i] = x;
		}
		nTokens++;
	}
	SetValidity();
}

Mask::Mask():
	m(0),
	valid(false),
	lenght(0),
	nTokens(0),
	nSub(0),
	nSuper(0)
{
}

Mask::Mask(Byte m):
	value(new Byte[m]),
	pFilled(new Byte[m]),
	m(m),
	valid(false),
	lenght(0),
	nTokens(0),
	nSub(0),
	nSuper(0)
{
	for(int i = 0; i < m; i++)
	  value[i] = GAP;
}

void Mask::Copy(const Mask& other)
{
	m = other.m;
	lenght = other.lenght;
	nTokens = other.nTokens;
	valid = other.valid;
	nSub = other.nSub;
	nSuper = other.nSuper;

	if(m > 0)
	{
		value = new Byte[m];
		for(int i = 0; i < m; i++)
	  	value[i] = other.value[i];
		pFilled = new Byte[m];
		for(int i = 0; i < nTokens; i++)
	  	pFilled[i] = other.pFilled[i];
	}

	if(nSub > 0)
	{
		pSub = new PMask[nSub];
		for(int i = 0; i < nSub; i++)
		{
			pSub[i].nMask = other.pSub[i].nMask;
			pSub[i].delta = other.pSub[i].delta;
		}
	}

	if(nSuper > 0)
	{
		pSuper = new PMask[nSuper];
		for(int i = 0; i < nSuper; i++)
		{
			pSuper[i].nMask = other.pSuper[i].nMask;
			pSuper[i].delta = other.pSuper[i].delta;
		}
	}
}

void Mask::Delete()
{
	if(m > 0)
	{
		delete [] value;
		delete [] pFilled;
	}

	if(nSub > 0)
		delete [] pSub;

	if(nSuper > 0)
		delete [] pSuper;

	m = 0;
	valid = false;
	nTokens = 0;
	lenght = 0;
	nSub = 0;
	nSuper = 0;
}


Mask::Mask(const Mask& other)
{
	Copy(other);
}


Mask::~Mask()
{
	Delete();
}

Mask::MRef Mask::operator[](Byte i)
{
	return Mask::MRef(*this, i);
}

Mask& Mask::operator=(const Mask& other)
{
	Delete();
	Copy(other);

	return *this;
}

Byte Mask::MDim() const
{
	return m;
}

Byte Mask::Lenght() const
{
	return lenght;
}

Byte Mask::NumOfTokens() const
{
	return nTokens;
}

Byte Mask::NumOfGaps() const
{
	return m - nTokens;
}


std::ostream& operator<<(std::ostream& output, const Mask& p)
{
	output << "(" << std::setw(3) << std::setfill('0') << static_cast<int>(p.lenght); 
	output << "/" << std::setw(3) << std::setfill('0') << static_cast<int>(p.nTokens);
	output << "/" << (p.valid? "o":"-");
	output << ") - ";

	for(int i = 0; i < p.m; i++)
	{
	 	if(p.value[i] == GAP)
	 		output << "_";
	 	else
	 	  output << "$";
	 	if(i < (p.m)-1) output << " ";
	};

	for(int i = 0; i < p.nSub; i++)
	{
		output << " {" << std::setw(3) << std::setfill('0') << static_cast<int>(p.pSub[i].nMask); 
		output << "/" << std::setw(3) << std::setfill('0') << static_cast<int>(p.pSub[i].delta);
		output << "}";
	};

	output << "\n                                    ";

	for(int i = 0; i < p.nSuper; i++)
	{
		output << " [" << std::setw(3) << std::setfill('0') << static_cast<int>(p.pSuper[i].nMask); 
		output << "/" << std::setw(3) << std::setfill(' ') << static_cast<int>(p.pSuper[i].delta);
		output << "]";
	};

	output << "\n                                    #";
	for(int i = 0; i < p.nTokens; i++)
		output << " " << std::setw(3) << std::setfill('0') << static_cast<int>(p.pFilled[i]); 

	return output;
}

bool Mask::operator==(const Mask& s) const
{
	bool res = true;
	if(lenght == s.lenght)
	{
		for(Byte i = first; i < lenght; i++)
	  	if(value[i] != s.value[i])
	  	{
	  		res = false;
	  		break;
	  	}
	}
	else
		res = false;
	return res;
}

bool Mask::operator>(const Mask& s) const
{
	Byte i;
	
	for(i = first; i < m; i++)
	  if(value[i] != s.value[i])
			break;
	return (i != m) && value[i] > s.value[i];
}

bool Mask::operator>=(const Mask& s) const
{
	Byte i;
	
	for(i = first; i < m; i++)
	  if(value[i] != s.value[i])
			break;
	return (i == m) || value[i] > s.value[i];
}

bool Mask::operator<(const Mask& s) const
{
	Byte i;
	
	for(i = first; i < m; i++)
	  if(value[i] != s.value[i])
			break;
  return (i!=m) && value[i] < s.value[i];
}

bool Mask::operator<=(const Mask& s) const
{
	Byte i;
	
	for(i = first; i < m; i++)
	  if(value[i] != s.value[i])
			break;
  return (i==m) || value[i] < s.value[i];
}

void Mask::SetValidity()
{
  bool result;

	short int meio = (m-1)/2;

	if(lenght == 1)
		result = false;
	else if(value[meio] != GAP)
	  result = true;
	else if(value[m-1] != GAP)
	  result = false;
	else
	{
		int i;
		int c2 = 0;
	  for(i = m-1; i>meio;i--)
	  {
	    if(value[i] != GAP) break;
	    c2++;
	  }
		int c1 = 0;
	  for(i = meio-1; i>=0;i--)
	  {
	    if(value[i] != GAP) break;
	    c1++;
	  }
	  result = (c2>c1);
	}
	valid = result;
}

bool Mask::Valid() const
{
	return valid;
}

void Mask::IniSub()
{
	if(nSub > 0)
	{
		delete [] pSub;
		nSub = 0;
	}
	pSub = new PMask[MaxSub()];
}

Byte Mask::MaxSub() const
{
	return nTokens;
}

void Mask::AddSub(short int nMask, short int delta)
{
	pSub[nSub].nMask = nMask;
	pSub[nSub++].delta = delta;
}

void Mask::IniSuper()
{
	if(nSuper > 0)
	{
		delete [] pSuper;
		nSuper = 0;
	}
	pSuper = new PMask[MaxSuper()];
}

Byte Mask::MaxSuper() const
{
	return(m - nTokens) + 2 * (m - lenght);
}

void Mask::AddSuper(short int nMask, short int delta)
{
	pSuper[nSuper].nMask = nMask;
	pSuper[nSuper++].delta = delta;
}


Byte Mask::ShiftLeft()      // Elimina primeiro Token        
{
	short int shift, i;
	for(shift = 1; ; shift++)
		if(value[shift] != GAP)
			break;

	for(i = shift; i < lenght; i++)
		value[i-shift] = value[i];

	for(i = lenght - shift; i < lenght; i++)
		value[i] = GAP;

	for(i = 0; i < nTokens-1; i++)
		pFilled[i] = pFilled[i+1]-shift;

	nTokens--;
	lenght -= shift;
	valid = valid && (lenght > 1);

	return shift;
}

void Mask::ShiftRight(bool clean)        // Acrescenta um Token a esquerda com GAPS
{
	if(clean)
		value[0] = GAP;
	for(int i = lenght; i >= 1; i--)
		value[i] = value[i-1];

	value[0] = FILLED;

	for(int i = 1; i <= nTokens; i++)
		pFilled[i] = pFilled[i-1]+1;

	pFilled[0] = 0;

	nTokens++;
	lenght++;
	SetValidity();

}

Byte Mask::NSub() const
{
	return nSub;
}

Byte Mask::NSuper() const
{
	return nSuper;
}

short int Mask::SubMask(Byte x) const
{
	return pSub[x].nMask;
}

short int Mask::SubDelta(Byte x) const
{
	return pSub[x].delta;
}

short int Mask::SuperMask(Byte x) const
{
	return pSuper[x].nMask;
}

short int Mask::SuperDelta(Byte x) const
{
	return pSuper[x].delta;
}

Byte Mask::Index(Byte i) const
{
	return pFilled[i];
}

