/*
 Copyright 2002, 2003 Alexandre Gil
                                                                                
 This file is part of mwu-extractor.
                                                                                
 mwu-extractor is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
                                                                                
 mwu-extractor is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
                                                                                
 You should have received a copy of the GNU General Public License
 along with mwu-extractor; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Nome: Extractor.cpp
// Objectivo: Extrator de MWUs. Corpo do programa principal.
//            Recebe um Corpus (conjunto de ficheiros binarios). Cada ficheiro contem uma cadeia
//            de Tokens (short int).  O Corpus deve ter um mximo de ULONG_MAX (4,294,967,295)
//            tokens. O Corpus  passado na forma dum ficheiro contendo a lista de ficheiros
//						binarios que o compoem.
//
//						Recebe ainda como paramentro a dimensao (m) da janela de analise que deve ser um
//            valor inteiro impar maior que 1 e menor que 256.
//
// Autor: Alexandre Gil
// Data: 13 Jan 2002

#include <iostream>
#include <cctype>
#include <iomanip>
#include <fstream>
#include <limits.h>

#include "MyUtils.h"
#include "Corpus.h"
#include "NGramMask.h"
#include "Matrix.h"

#include "Mask.h"
#include "NGram.h"
#include "ISVector.h"


int main(int argc, char *argv[])
{
	std::cout << "Sizeof Pos= " << sizeof(Corpus::Pos) << std::endl;
	std::cout << "Sizeof Mask= " << sizeof(Mask) << std::endl;
	std::cout << "Sizeof MNode= " << sizeof(MNode) << std::endl;
	std::cout << "Sizeof NGram= " << sizeof(NGram) << std::endl;
	std::cout << "Sizeof Token= " << sizeof(Token) << " max= " <<  UINT_MAX << std::endl;

	std::cout << std::endl << "Extractor de MWUs (Ideal, V1)\n";
	
  if(argc == 4)
	{
		int m = str2int(argv[1]);
		if((m % 2) == 1 && m > 1 && m < 256)     // m tem que ser impar, maior que 1 e menor que 256
		{
			ClearCRLF(argv[3]);
			std::ofstream fout(argv[3], std::ios::binary);
			if(!fout)
			{
			  std::cout << "ERRO: Falhou inicializacao do ficheiro output. Verifique ficheiro " << argv[3] << ".\n";
			  return 1;
			}
			else
			{
				ClearCRLF(argv[2]);
				TimeStamp(); std::cout << " Ler ficheiro " << argv[2] << ". Iniciar Corpus." << std::endl;
			  Corpus c(argv[2]);
				if(!c.Erro())
				{
					if((int)c.Dim() > m)
					{
						TimeStamp(); std::cout << " Iniciar vector de mascaras." << std::endl;
						NGramMask masks(m, true);			// Criar vector mascara para geracao dos n-gramas
//  				masks.Write();

						TimeStamp(); std::cout << " Iniciar Matrix de " << c.Dim() << " por " << masks.NumOfMasks() << "=" << c.Dim()*masks.NumOfMasks() << std::endl;
  					Matrix matrix(c, masks);
//  				std::cout << matrix;
					
						TimeStamp(); std::cout << " Criar ficheiro de MWUs\n";
						matrix.WriteMWUs(fout, true);
						fout.close();					
				    TimeStamp(); std::cout << " Terminado" << std::endl;

			 	    return 0;
					}
				 	else
				 	 	std::cout << "Corpus " << argv[2] << " vazio ou tem menos de " << m << " elementos" << std::endl;
				}
				else
				{
				   std::cout << "ERRO: Falhou inicializacao do Corpus. Verifique ficheiro " << argv[2] << ".\n";
				   return 1;
				}
			}
		}
		else
		{
		  std::cout << "ERRO: m (" << m << ") deve ser impar, maior que 1 e menor que 256.\n";
		  return 1;  	
		}
	}
	else
	{
    std::cout << "ERRO: Numero errado de argumentos. ex <m> <Corpus> <Output>.\n";
    return 1;
  }
}

