/*
 Copyright 2002, 2003 Alexandre Gil
                                                                                
 This file is part of mwu-extractor.
                                                                                
 mwu-extractor is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
                                                                                
 mwu-extractor is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
                                                                                
 You should have received a copy of the GNU General Public License
 along with mwu-extractor; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Nome: Matrix.cpp
// Objectivo: Implementacao da classe Matrix.
// Autor: Alexandre Gil
// Data: 08 Fev 2002

#include "Matrix.h"
#include "Corpus.h"
#include "NGramMask.h"
#include "NGram.h"
#include "ISVector.h"
#include "MyUtils.h"
#include "Globals.h"

#include <iomanip>

Matrix::Matrix(const Corpus& c, const NGramMask& masks):
	c(c),
	masks(masks),
  dummy(new MNode)
{
	nMasks = masks.NumOfMasks();
	cDim = c.Dim();
	nFiles = c.NumOfFiles();
	
	dummy->GrowTo(1);
	dummy->Add(Corpus::Pos(0,0));

	value = new PMNode[cDim*nMasks];

	InitMatrix();

	lastMask = -1;			// basta para obrigar a procurar
}

Matrix::~Matrix()
{
	::PMNode a;
  unsigned long freq, f;

	for(Corpus::Pos x = c.FirstPos(); x <= c.LastPos(); x.Next(c))
		for(short int y = 0; y < nMasks; y++)
		{
		 	a = At(x,y);
  		if(a != 0 && a != dummy)
		  {
		  	freq = a->Freq();
		  	for(f = 0; f < freq; f++)
					PutAt(a->Pos(f), y, 0);
				delete a;
		  }
		}
	delete dummy;
	delete [] value;
}

std::ostream& operator<<(std::ostream& output, const Matrix& p)
{
	p.Write(output,false);
	return output;
}

void Matrix::Write(std::ostream& output, const bool onlyCount) const
{
	unsigned long nulos = 0;
	unsigned long dummys = 0;
	unsigned long filled = 0;

	float me;

	for(short int  y = 0; y < nMasks; y++)
		for(Corpus::Pos x = c.FirstPos(); x <= c.LastPos(); x.Next(c))
			if(At(x,y) == 0)
			{
				if(!onlyCount) output << "Matrix[" << x << "," << y << "] NULL" << std::endl;
				nulos++;
			}
			else if (At(x,y) == dummy)
			{
				me = CalcME(x, y);
				if(!onlyCount)
				{
					output << "Matrix[" << x << "," << y << "] ";
					output << std::setw(16) << std::setprecision(14) << std::setfill('0') << std::setiosflags(std::ios::fixed) << me;
					output << " DUMMY " << std::endl;
				}
				dummys++;
			}
			else
			{
				me = CalcME(x, y);
				if(!onlyCount)
				{
					output << "Matrix[" << x << "," << y << "] ";
					output << std::setw(16) << std::setprecision(14) << std::setfill('0') << std::setiosflags(std::ios::fixed) << me;
					output << " " << (*(At(x,y))) << std::endl;
				}
				filled++;
			}

	output << "Dummys= " << dummys << " Nulos= " << nulos << " filled= " << filled << " TOTAL " << (filled+nulos+dummys) << std::endl;
}

void Matrix::PutAt(const Corpus::Pos& x, const short int y, const ::PMNode pn)
{
	value[c.Index(x)* nMasks + y] = pn;
}

::PMNode Matrix::At(const Corpus::Pos& x, const short int y) const
{
	return value[c.Index(x) * nMasks + y];
}

void Matrix::InitMatrix()
{
  NGram::SetBase(&c,&masks);
  MNode::SetCorpus(&c);

	Corpus::Pos p1;
	short int z;

	Token actual;
	Byte nTokens;
  unsigned long i, j, x;
	NGram ngAnte, ngActual;
  Token previous;
  unsigned long ini;

	for(Corpus::Pos x = c.FirstPos(); x <= c.LastPos(); x.Next(c))
		for(short int y = 0; y < nMasks; y++)
		{
			ngActual = NGram(x,y);
			if(ngActual.Valid())
	  		PutAt(x,y,dummy);
	  	else
	  		PutAt(x,y,0);
		}

 	ISVector<NGram> ngv(cDim);									// Vector auxiliar

	Corpus::Pos p = c.FirstPos();								// preencher com todas as posicoes do Corpus
	for(j = 0; j < cDim; j++)
	{
		ngv.PutAt(j, new NGram(p, 0));
		p.Next(c);
	}

	TimeStamp(); std::cout << " Ordenar ngv zero" << std::endl;
	ngv.Sort(0, cDim-1);													// ordenar Corpus
	TimeStamp(); std::cout << " Ordenado ngv zero. cDim=" << cDim << " ngv.Filled()=" << ngv.Filled() << std::endl;

	i = 0;																	// ponto de insercao
	j = 0;																	// contador
	ini = 0;																// inicio de cada ciclo

	NGram::SetFirst(0);
	while(j<ngv.Filled())														// Eliminar Tokens com Freq=1
	{
		do
			j++;
		while( (j < ngv.Filled()) && (*(ngv.At(j)) == *(ngv.At(ini))));
		if((j - ini) == 1)
			delete ngv.At(ini);
		else if(ini != i)
			for(x = ini; x < j; x++)
				ngv.PutAt(i++,ngv.At(x));
		else
			i = j;
		ini = j;
	}
	ngv.Reset(i);
	TimeStamp(); std::cout << " Comprimido ngv zero. Total Tokens com freq > 1= " << ngv.Filled() << std::endl;
	
	totalNGrams = 0;
	totalMNodes = 0;
	totalFilled = 0;
	invalid = 0;


	FillMatrix(0, ngv);

	for(z = 1; z < nMasks; z++)
	{
  	ini = 0;
	  previous = c.At(ngv.At(0)->Pos());
		nTokens = masks.At(z)->NumOfTokens();
		for(j = 1; j < ngv.Filled(); j++)
		{
			ngv.At(j)->IncMask(); 										 // Actualiza Mask
			actual = c.At(ngv.At(j)->Pos());
			if (actual != previous)
			{
				if((j - ini) > 1)
					ngv.Sort(ini, j-1, 1, nTokens-1);
				ini = j;
			}
			previous = actual;
		}
		if((j - ini) > 1)
			ngv.Sort(ini, j-1, 1, nTokens-1);
		FillMatrix(z, ngv);
	}

	for(j = 0; j < ngv.Filled(); j++)         // Apagar ISVector<NGram>
		delete ngv.At(j);

	TimeStamp(); std::cout << " Total n-gramas = " <<  totalNGrams << std::endl;
	TimeStamp(); std::cout << " Total n-gramas diferentes com freq > 1 (MNodes) = " << totalMNodes << std::endl;
	TimeStamp(); std::cout << " Total n-gramas com freq > 1 (Filled) = " << totalFilled << std::endl;
	TimeStamp(); std::cout << " Total de invalidos = " << invalid << std::endl;
}

void Matrix::FillMatrix(const short int mask, const ISVector<NGram>& ngv)
{
	unsigned long numberOf = ngv.Filled();
	unsigned long r;                       // Numero de repeticoes em cada loop
	unsigned long j;
	
	::PNGram p;                   				 // Pointer para o NGram no inicio dum ciclo
	::PMNode pNode;												 // Pointer para novo MNode

	if(numberOf > 0)
	{
		r = 0;
		NGram::SetFirst(0);
	  for(unsigned long i = 1; i <= numberOf; i++)
	  {
	  	if(i < numberOf && (*(ngv.At(i)) == *(ngv.At(i-1))))
	  		r++;
	  	else
	  	{
 				r++;
				totalNGrams += r;
	  		if(!ngv.At(i-r)->Valid())
  				invalid += r;
  			else if(r > 1)
  			{
					totalMNodes++;
	 				pNode = new MNode();
	 				pNode->GrowTo(r);
	  			for(j = i-r; j < i; j++)
	  			{
	  				p = ngv.At(j);
						PutAt(p->Pos(), mask, pNode);
						totalFilled++;
		 				pNode->Add(p->Pos());
		 			}
  			}
				r = 0;
	  	}
		}
	}
}

const ::PMNode Matrix::Dummy() const
{
	return dummy;
}

float Matrix::CalcME(const Corpus::Pos& pos, const short int nMask) const
{
	const unsigned long freq = At(pos,nMask)->Freq();
	float me = 0;
	unsigned long soma;
	Byte nSub;
	short int delta;
	Corpus::Pos pAux;

	if(masks.At(nMask)->Lenght() > 1)
	{
		nSub = masks.At(nMask)->NSub();
		if(nSub > 0)
		{
			soma = 0;
			pAux = pos;
			for(Byte mk = 0; mk < nSub; mk++)
			{
				delta = masks.At(nMask)->SubDelta(mk);
				if(delta != 0)
				{
					pAux.MoveSameFile(delta);
					soma += At(pAux, masks.At(nMask)->SubMask(mk))->Freq();
					pAux.MoveSameFile(-delta);
				}
				else
				{
					soma += At(pAux, masks.At(nMask)->SubMask(mk))->Freq();
				}
			}
			if(soma > 0)
				me = (static_cast<float>(freq) * static_cast<float>(freq)) / ((static_cast<float>(cDim) / static_cast<float>(nSub)) * static_cast<float>(soma));
		}
	}
	return me;
}

bool Matrix::IsLocalMax(const Corpus::Pos pos, const short int nMask) const
{
	PMNode pNode = At(pos, nMask);
	Corpus::Pos pAux;
	Byte mk;
	
	short int delta;
	float maxSub;
	float maxSuper;
	short int superMask;

	unsigned long f, freq;

	bool res;

	float me = CalcME(pos, nMask);

	maxSub = 0;
	if(masks.At(nMask)->NumOfTokens() > 2)
	{
		pAux = pos;
		for(mk = 0; mk < masks.At(nMask)->NSub(); mk++)
		{
			delta = masks.At(nMask)->SubDelta(mk);
			if(delta != 0)
			{
				pAux.MoveSameFile(delta);
				maxSub = MAX(maxSub, CalcME(pAux, masks.At(nMask)->SubMask(mk)));
				pAux.MoveSameFile(-delta);
			}
			else
				maxSub = MAX(maxSub, CalcME(pAux, masks.At(nMask)->SubMask(mk)));
		}
	}

	maxSuper = 0;
	freq = pNode->Freq();
	for(f = 0; f < freq; f++)
	{
		pAux = pNode->Pos(f);  
		for(mk = 0; mk < masks.At(nMask)->NSuper(); mk++)
		{
			superMask = masks.At(nMask)->SuperMask(mk);
			delta = masks.At(nMask)->SuperDelta(mk);
			if(delta != 0)
			{
				if(pAux.CanMoveSameFile(c, delta))
				{
					pAux.MoveSameFile(delta);
					if(At(pAux, superMask) != 0)    // testa validade do super-grama 
						maxSuper = MAX(maxSuper, CalcME(pAux, superMask));
					pAux.MoveSameFile(-delta);
				}
			}
			else
			{
				if(At(pAux, superMask) != 0)    // testa validade do super-grama 
					maxSuper = MAX(maxSuper, CalcME(pAux, superMask));
			}
		}
	}
  res = (maxSub <= me) && (me > maxSuper);
	return res;
}

void Matrix::ResetAllNodes()
{
	::PMNode a;
	
	for(Corpus::Pos x = c.FirstPos(); x <= c.LastPos(); x.Next(c))
		for(short int y = 0; y < nMasks; y++)
		{
			a = At(x,y);		
			if(a != 0 && a != dummy)
		  	a->Reset();
		}
}

void Matrix::WriteMWUs(std::ofstream& fout, const bool restrict)
{
	PMNode pNode;
	float me;
	Byte m;
	long freq;

	float gerados = 0;

	TimeStamp(); std::cout << "Vai desmarcar todos os nos da Matrix" << std::endl;
	ResetAllNodes();
	TimeStamp(); std::cout << "Fim desmarcar todos os nos da Matrix" << std::endl;
	
	for(short int y = 1; y < nMasks; y++)     // Excluida a mascara zero
	{
		std::cout << "                                  \r";
		std::cout << gerados << "\t" << y << "\r";
		if(masks.At(y)->NumOfGaps() > 0) 								// && (!restrict || restrict && masks.At(y)->Valid()))	
			for(Corpus::Pos x = c.FirstPos(); x <= c.LastPos(); x.Next(c))
			{
	  		pNode = At(x,y);
	 			if(pNode != 0 && pNode != dummy && !pNode->IsSet())
	 			{
					if(IsLocalMax(x, y))
					{
						m = masks.At(y)->Lenght();
						fout.write((char *)&m, sizeof(Byte));					
	  				me = CalcME(x,y);
						fout.write((char *)&me, sizeof(float));
						freq = pNode->Freq();
						fout.write((char *)&freq, sizeof(long));
						WriteNGram(fout, x, y);
	//  					std::cout << "  " <<std::setw(16) << std::setprecision(14) << std::setfill('0') << std::setiosflags(std::ios::fixed) << me;
	//  					std::cout << " Freq= " << freq << " m= " << (int)m << std::endl;
						gerados++;
					}
					pNode->Set();
				}
			}
	}
	std::cout << "                                  \r";
	std::cout << "Total MWUs= " << gerados << std::endl;
}

void Matrix::WriteNGram(std::ostream& output, const Corpus::Pos pos, const short int nMask) const
{
	Token tk;
	short int k = masks.At(nMask)->Lenght();
	Corpus::Pos p = pos;
	for(int i = 0; i < k; i++)
	{
		if(masks.At(nMask)->At(i) == GAP)
			tk = 0;
		else
			tk = c.At(p);
		output.write((char *)&tk, sizeof(Token));
		p.Next(c);
	}
}


