/*
 Copyright 2002, 2003 Alexandre Gil
                                                                                
 This file is part of mwu-extractor.
                                                                                
 mwu-extractor is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
                                                                                
 mwu-extractor is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
                                                                                
 You should have received a copy of the GNU General Public License
 along with mwu-extractor; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Nome: NGram.h
// Objectivo: Definicao das Classes NGram e MNode.
// Pressupostos: Os n-gramas tem dimensao inferior a 256 (1 byte)
//							 O NGram serve para representar n-gramas durante ordenacao
//               Os VNode + MNode servem para representar n-gramas quando associados a Matrix
// Autor: Alexandre Gil
// Data: 01 Fev 2002
//			 14 Abr 2002

#ifndef _H_NGram
#define _H_NGram


#include "Corpus.h"
#include "NGramMask.h"
#include "MyUtils.h"

class NGram {
	private:
		Corpus::Pos pos;
		short int pMask;

		static Corpus const * pCorpus;
		static NGramMask const * pvMask;
		static Byte first;
	private:
		bool GreaterThan(const NGram& other, bool restrict) const;
		bool LessThan(const NGram& other, bool restrict) const;
	public:
		NGram();
		NGram(const Corpus::Pos& pos, const short int pMask);
		NGram(const NGram& other);
		~NGram();

		NGram& operator=(const NGram& other);

		static void SetBase(const Corpus * c, const NGramMask * vMask);
		static void SetFirst(const Byte firstToken);

		bool Valid() const;
		void IncMask();
		
		bool operator==(const NGram& other) const;
		bool operator>(const NGram& other) const;
		bool operator<(const NGram& other) const;
		bool operator>=(const NGram& other) const;
		bool operator<=(const NGram& other) const;

		friend std::ostream& operator<<(std::ostream& output, const NGram& p);

		const Corpus::Pos& Pos() const;
		short int PMask() const;

		Byte Lenght() const;
		Byte NumOfTokens() const;
		Byte NumOfGaps() const;

		Token At(const Byte i) const;  // i < pvMask->At(pMask).NumOfTokens()
};


typedef	NGram* PNGram;

class MNode {
	private:
		Corpus::Pos* pos;         // Vector de posicoes com igual n-grama
		long freq;								// frequencia num devera ser maior que 1/2 do maximo n-gramas
															// sinal e' utilizado para "marcar" objecto
		static Corpus const * pCorpus;
	public:
		MNode();
		MNode(const Corpus::Pos& p);
		MNode(const MNode& other);

		~MNode();

		MNode& operator=(const MNode& other);

		static void SetCorpus(const Corpus * c);

		friend std::ostream& operator<<(std::ostream& output, const MNode& p);

		void GrowTo(long dim);
		void Add(const Corpus::Pos& p);

		const Corpus::Pos& Pos(long f) const;

		long Freq() const;

		void Set();
		void Reset();
		bool IsSet() const;
};



typedef	MNode * PMNode;

#endif
