/*
 Copyright 2002, 2003 Alexandre Gil
                                                                                
 This file is part of mwu-extractor.
                                                                                
 mwu-extractor is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
                                                                                
 mwu-extractor is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
                                                                                
 You should have received a copy of the GNU General Public License
 along with mwu-extractor; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Nome: Matrix.h
// Objectivo: Interface da classe generica Matrix
// Autor: Alexandre Gil
// Data: 08 Fev 2002

#ifndef _H_Matrix
#define _H_Matrix

#include "Corpus.h"
#include "NGramMask.h"
#include "NGram.h"
#include "ISVector.h"

class Matrix {
	private:
		union MatrixNode {::PMNode pNode; long freq;};
	private:
		MatrixNode * value;
		const Corpus& c;
		const NGramMask& masks;

		unsigned long cDim;
		short int nMasks;
		unsigned short nFiles;

		unsigned long totalNGrams;							// Numero de n-gramas inseridos na Matrix
		unsigned long totalMNodes;							// Numero de MNODES criados (n-gramas diferentes com freq > 1)
		unsigned long totalPos;									// Numero de Pos associados a MNODES criados (n-gramas diferentes com freq > 1)
		unsigned long totalFilled;							// Numero de quadriculas preenchidas com valor != 0 e != dummy
		unsigned long totalFilledZero;					// Numero de quadriculas da mask zero preenchidas com freq > 1
		unsigned long invalid;									// Numero de n-gramas invalidos

		const ::PMNode dummy;
	private:
		void Write(std::ostream& output, const bool onlyCount) const;
		void InitMatrix();
		void FillMatrix(const short int mask, const ISVector<NGram>& ng);

		float CalcME(const Corpus::Pos& pos, const short int nMask) const;
  		// e.g. ME(w1 w2 w3) = ( f(w1 w2 w3)/T ) * f(w1 w2 w3) / (1/3 * (f(w1 w2) + f(w1 w3) + f(w2 w3)))
  		// e.g. ME(w1 w2) = (f(w1 w2) / T) * ( (2*f(w1 w2)) / (f(w1) + f(w2)) )  

		void Matrix::PutAt(const Corpus::Pos& x, const short int y, const ::PMNode pNode);
		void Matrix::PutAt(const Corpus::Pos& x, const short int y, const long freq);
		void ResetAllNodes();

	  bool IsLocalMax(const Corpus::Pos pos, const short int nMask) const; // pos,nMask define um n-grama
			// w  uma MWU se e s para qualquer x pertencente ao SubSet(w) and
		  //                     para qualquer y pertencente ao SuperSet(w):
			// (NumOfTokens(w) = 2 and g(w) > g(y) ) or 
			// (NumOfTokens(w) > 2 and g(x) <= g(w) and g(w) > g(y) )

		::PMNode At(const Corpus::Pos& x, const short int y) const;
		long FreqAt(const Corpus::Pos& x, const short int y) const;
		
		const PMNode Matrix::Dummy() const;

		void WriteNGram(std::ostream& output, const Corpus::Pos pos, const short int nMask) const;
	public:
		Matrix(const Corpus& c, const NGramMask& masks);
		~Matrix();

 		friend std::ostream& operator<<(std::ostream& output, const Matrix& p);

	  void WriteMWUs(std::ofstream& fout, const bool restrict);		
};


#endif

