/*
 Copyright 2002, 2003 Alexandre Gil
                                                                                
 This file is part of mwu-extractor.
                                                                                
 mwu-extractor is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
                                                                                
 mwu-extractor is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
                                                                                
 You should have received a copy of the GNU General Public License
 along with mwu-extractor; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Nome: TCorpus.cpp
// Objectivo: Implementacao da abstraccao para ficheiros do TCorpus
// Autor: Alexandre Gil
// Data: 16 Mar 2002

#include <fstream>
#include <cstring>

#include "TCorpus.h"

#include <iostream>
#include <iomanip>
#include <stdexcept>


void TCorpus::ReadFileName(std::ifstream& fList, char * n, int max)
{
    char c;

    int i = 0;
    c = fList.get();
    while(c != '\n' && c != '\r' && !fList.eof() && i < max)
    {
	n[i++] = c;
	c = fList.get();
    }
    n[i] = '\0';
    while(c != '\n' && !fList.eof())
	c = fList.get();
}

TCorpus::TCorpus():
    numFiles(0),
    erro(false)
{
}

TCorpus::TCorpus(const char * fLName):
    corpusName(new char[strlen(fLName)+1]),
    numFiles(0),
    erro(false)
{
    strcpy(corpusName, fLName);
    
    char buf[MAXFILENAME];
    
    std::ifstream fList(corpusName);
    erro = !fList;
    if (!erro) {
	ReadFileName(fList, buf, MAXFILENAME);
	while(!erro && !fList.eof() && numFiles < MAXFILES) {
	    if(strlen(buf) > 0) {
		file[numFiles].name = new char[strlen(buf)+1];
		strcpy(file[numFiles].name, buf);
		currentFile = numFiles;
		OpenCurrentFile();
		if(!erro) {
		    fp.close();
		    numFiles++;
		}
	    }
	    if(!fList.eof())
		ReadFileName(fList, buf, MAXFILENAME);
	}
	fList.close();
	// erro = erro || !fList;
	if(!erro && numFiles > 0) {
	    currentFile = 0;
	    OpenCurrentFile();
	}
    }
}

TCorpus::~TCorpus()
{
    for(int i = 0; i<numFiles; i++)
	delete [] file[i].name;
    fp.close();
    delete [] corpusName;
}

bool TCorpus::Erro() const
{
    return erro;
}

bool TCorpus::AtEOF() const
{
    return fp.eof();
}

unsigned char TCorpus::Current() const
{
    return current;
}

void TCorpus::NextChar()
{
    current = fp.get();
    erro = !fp;
}

int TCorpus::NumOfFiles() const
{
    return numFiles;
}

void TCorpus::OpenCurrentFile()
{
    fp.open(file[currentFile].name);
    erro = !fp;
    if(!erro)
	NextChar();
}

void TCorpus::NextFile()
{
    fp.close();
    currentFile++;
    if(currentFile != numFiles)
	OpenCurrentFile();
}

const char * TCorpus::ExtractName(const char * fullPathName) const
{
    int i;
    for(i=strlen(fullPathName);i >= 0; i--)
	if(fullPathName[i] == '\\' || fullPathName[i] == '/' || fullPathName[i] == ':')
	    break;
    return &fullPathName[i+1];
}

const char * TCorpus::FileName() const
{
    return ExtractName(file[currentFile].name);
}

const char * TCorpus::CorpusName() const
{
    return ExtractName(corpusName);
}

bool TCorpus::AtEndOfCorpus() const
{
    return currentFile == numFiles;
}
