/*
 Copyright 2002, 2003 Alexandre Gil
                                                                                
 This file is part of mwu-extractor.
                                                                                
 mwu-extractor is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
                                                                                
 mwu-extractor is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
                                                                                
 You should have received a copy of the GNU General Public License
 along with mwu-extractor; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Nome: TSTree.cpp
// Objectivo: Implementacao para a classe TSTree
//            Arvore ternaria de strings
// 						Baseado no codigo de Jon Bentley & Bob Sergerwick
//            - Dr. Dobb's Journal April 1998
//						- Fast Algorithms for Sorting and Searching Strings, Jan 1997
//            
// Autor: Alexandre Gil
// Data: 17 Mar 2002

#include <iostream>
#include <iomanip>
#include <cstring>

#include "MyUtils.h"
#include "TSTree.h"

template<class T>
TSTree<T>::TSTree(const T count):
    count(count)
{
    for(int i=0;i<256;i++)
	root[i] = 0;
}

template<class T>
void TSTree<T>::DeleteNode(const TNode<T>* node)
{
    if(node != 0)
    {
	DeleteNode(node->lokid);
	DeleteNode(node->eqkid);
	DeleteNode(node->hikid);
	delete node;
    }
}
	
template<class T>
TSTree<T>::~TSTree()
{
    for(int i=0;i<256;i++)
	DeleteNode(root[i]);
}

template<class T>
TNode<T>* TSTree<T>::_Insert(const char * s, TNode<T>* node, int& v)
{
    TNode<T>* p = node;

    if(p == 0)                                    // No encontrou elemento 
    {
	p = new TNode<T>;                           // Cria novo n.
	p->lokid = p->eqkid = p->hikid = 0;
	if( (p->splitchar = *s) == '\0')            // Insere e stop se for fim de string
	    v = p->value = count++;										// Atribui valor para novo elemento
	else
	    p->eqkid = _Insert(++s, p->eqkid, v);     // Insere proximo caracter
    }
    else if(*s < p->splitchar)										// Elemento maior. Vai pela esquerda
	p->lokid = _Insert(s, p->lokid, v);	
    else if(*s > p->splitchar)										// Elemento menor. Vai pela direita
	p->hikid = _Insert(s, p->hikid, v);
    else if(*s == '\0')														// Elemento igual. 
	v = p->value;																// Se '\0' elemento j existente. Devolve valor.
    else
	p->eqkid = _Insert(++s, p->eqkid, v);       // Avana pelo centro

    return p;
}

template<class T>
T TSTree<T>::Insert(const char * s)
{
    int res;
    unsigned char c = *s;

    root[c] = _Insert(++s, root[c], res);

    return res;
}

template<class T>
void TSTree<T>::WriteNode(std::ostream& output, const char * prefix, const TNode<T>* node) const
{
    char * pref;
	
    if(node != 0)
    {
	WriteNode(output, prefix, node->lokid);
	if(node->splitchar == '\0')
	    output << std::setw(10) << std::setfill('0') << node->value << "\t" << prefix << std::endl;
	int l = strlen(prefix);
	pref = new char[l+2];
	strcpy(pref, prefix);
	pref[l] = node->splitchar;
	pref[l+1] = '\0';
	WriteNode(output, pref, node->eqkid);
	delete [] pref;
	WriteNode(output, prefix, node->hikid);
    }
}

template<class T>
void TSTree<T>::Write(std::ostream& output)
{
    char c[2] = " ";
	
    for(unsigned char i=0; i<255;i++)
    {
	c[0] = i;
	WriteNode(output, c , root[i]);
    }
}

template<class T>
T TSTree<T>::Count() const
{
    return count;
}
