/*
 Copyright 2002, 2003 Alexandre Gil
                                                                                
 This file is part of mwu-extractor.
                                                                                
 mwu-extractor is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
                                                                                
 mwu-extractor is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
                                                                                
 You should have received a copy of the GNU General Public License
 along with mwu-extractor; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Nome: Translator.cpp
// Objectivo: Tradutor. Programa simples de conversao de binario para texto.
//            Recebe um ficheiro de tokens agrupados em MWUs no formato do output
//            do Extractor e um Dicionario.
//
//						Devolve o ficheiro "traduzido" segundo o dicionario.
//
//            Formato do ficheiro de entrada:
//						==============================
//						 um ficheiro binario com records variaveis com o seguinte formato:
//							Byte m 					- Lenght do MWUs
//							float me				-	Valor da Expectativa Mtua
//							long freq				-	Frequencia
//							Token token_1
//							Token token_2
//              ....
//     					Token token_m
//							
//
//						Formato do ficheiro do dicionario:
//						=================================
//						 uma ficheiro de texto com duas colunas separadas por ' ' ou '\t'
//            Primeira coluna  o Token a segunda  a string correspondente
//						A primeira linha tem somente uma coluna com o nmero total de tokens
//
//						Formato do ficheiro de sada:
//						============================
//						Ficheiro de texto equivalente ao ficheiro de entrada, orientado a linha e
// 						com o seguinte formato em cada linha:
//							float me			-	Valor da Expectativa Mtua
//							char * token_1
//							char * token_2
//              ....
//     					char * token_m
//							
//						
//
//
// Autor: Alexandre Gil
// Data: 30 Mar 2002

#include <iostream>
#include <cctype>
#include <iomanip>
#include <fstream>
#include <cstring>

#include "MyUtils.h"
#include "Token.h"
#include "Dictionary.h"

const char * EXT = ".txt";

int main(int argc, char *argv[])
{
	bool erro;
	bool IsInputFile;
	
	std::cout << "Translator\n";
	
  if(argc == 4)
	{
		if(argv[1][0] == '-' && argv[1][1] == 'I')
			IsInputFile = true;
		else if(argv[1][0] == '-' && argv[1][1] == 'O')
			IsInputFile = false;
		else
		{
    	std::cout << "ERRO: Primeiro argumento " << argv[1] << " nao valido" << std::endl;
    	return 1;
		}
		ClearCRLF(argv[2]);
		TimeStamp(); std::cout << " Abrir ficheiro " << argv[2] << std::endl;
		std::ifstream f(argv[2], std::ios::binary);
		erro = !f;
		if(!erro)
		{
			ClearCRLF(argv[3]);
			TimeStamp(); std::cout << " Ler dicionario " << argv[3] << std::endl;
			Dictionary dic(argv[3]);
			if(!dic.Error())
			{
//				dic.Write();
				TimeStamp(); std::cout << " Traduzir ficheiro " << argv[2] << std::endl;

				char * filename = new char[strlen(argv[2])+strlen(EXT)+1];
				strcpy(filename, argv[2]);
				strcat(filename, EXT);
				
				std::ofstream fout(filename);
				erro = !fout;
				if(!erro)
				{
					Byte m, i;
					float me;
					long freq;
					unsigned int tk;

					if(IsInputFile)
					{
						unsigned int i = 0;
						
						while(!erro && !f.eof())
						{
							f.read((char *)&tk, sizeof(tk));
							if(f.eof())
								break;
							erro = !f;
 							if(erro)
								break;
							else if(tk == 0)
								fout << " <gap>";
							else if(dic.Valid(tk))
								fout << i << "\t\t" << dic.String(tk) << std::endl;
							else
							{
								erro = true;
							 	std::cout << "ERRO: Token " << tk << " invalido." << std::endl;								
								break;
							}
							i++;
						}
						
					}
					else
					{
						while(!erro && !f.eof())
						{
							f.read((char *)&m, sizeof(Byte));
							if(f.eof())
								break;
							erro = !f;
							if(!erro)
							{
								f.read((char *)&me, sizeof(float));
								erro = !f || f.eof();
								if(!erro)
								{
									fout << std::setw(16) << std::setprecision(14) << std::setfill('0') << std::setiosflags(std::ios::fixed) << me;

									f.read((char *)&freq, sizeof(long));
									erro = !f || f.eof();
									if(!erro)
									{
										fout << " " << std::setw(5) << std::setfill('0') << freq;
										for(i = 0; i < m; i++)
										{
											f.read((char *)&tk, sizeof(tk));
											erro = !f || f.eof();
											if(erro)
												break;
											else if(tk == 0)
												fout << " <gap>";
											else if(dic.Valid(tk))
												fout << " " << dic.String(tk);
											else
											{
												erro = true;
									    	std::cout << "ERRO: Token " << tk << " invalido." << std::endl;								
												break;
											}
										}
									}
									if(!erro)
										fout << std::endl;
								}
							}
						}
					}
					if(erro)
			    	std::cout << "ERRO: Falhou leitura/traducao do ficheiro " << argv[2] << std::endl;
					else
					{
						fout.close();
				    TimeStamp(); std::cout << " Ficheiro " << filename << " criado com sucesso." << std::endl;
					}
				}
				else
		    	std::cout << "ERRO: Falhou criacao do ficheiro " << filename << std::endl;
				delete filename;
				if(erro)
					return 1;
				else
					return 0;
		  }
			else
		 	{
		    std::cout << "ERRO: Falhou incializacao do Dicionario " << argv[3] << std::endl;
	  	  return 1;
		 	}
		}
		else
	 	{
	    std::cout << "ERRO: Falhou abertura do ficheiro " << argv[2] << std::endl;
  	  return 1;
	 	}
  }
	else  	
  {
    std::cout << "ERRO: Numero errado de argumentos. tl {-I|-O} <file> <dic>." << std::endl;
    return 1;
  }
}

